Imports System.ComponentModel
Imports System.ComponentModel.Design
''' <summary>
''' Designer fr einen AlignContainer
''' </summary>
Public Class AlignContainerDesigner
    Inherits System.Windows.Forms.Design.ParentControlDesigner

#Region " Private Variablen "

    ' Flag, mit Information, ob sich die Maus ber dem Control befindet
    Private m_mouseover As Boolean = False
    ' Diese Farbe ist ein privates Feld fr die OutlineColor-Eigenschaft
    Private m_OutlineColor As Color = Color.White
    ' Referenz zur Aktionsliste fr den SmartTag
    Private m_meineAktionen As DesignerActionListCollection
    ' Referenz zum aktuell bearbeiteten AlignContainer
    Private m_AlignContainer As AlignContainer
    ' Pen zum Zeichnen von Inhalten
    Private m_DotPen As Pen

#End Region

#Region " Properties "

    ''' <summary>
    ''' Referenz zur Aktionsliste mit Aktionen, die der SmartTag beinhalten soll
    ''' </summary>
    Public Overrides ReadOnly Property ActionLists() As System.ComponentModel.Design.DesignerActionListCollection
        Get
            If m_meineAktionen Is Nothing Then
                ' Aktionsliste wird bei Bedarf einmalig als Objekt instanziiert
                m_meineAktionen = New DesignerActionListCollection
                Dim Liste As New AlignContainerDesignerActionList(CType(m_AlignContainer, Control))
                m_meineAktionen.Add(Liste)
            End If
            Return m_meineAktionen
        End Get
    End Property

    ''' <summary>
    ''' Diese Farbe wird verwendet, um das Control im MouseOver-Zustand mit weiem Rahmen zu markieren
    ''' </summary>
    Public Property OutlineColor() As Color
        Get
            Return m_OutlineColor
        End Get
        Set(ByVal Value As Color)
            m_OutlineColor = Value
        End Set
    End Property

#End Region

#Region " Konstruktor "

    Public Sub New()
        ' Pen instanziieren
        m_DotPen = New Pen(Brushes.Lime, 2)
        m_DotPen.DashStyle = Drawing2D.DashStyle.Dot
    End Sub

#End Region

#Region " Methoden "

    ''' <summary>
    ''' Initialisiert den Designer.
    ''' </summary>
    ''' <param name="component">Zugehrige Komponente.</param>
    Public Overloads Overrides Sub Initialize(ByVal component As IComponent)
        ' Basisklasse initialisieren
        MyBase.Initialize(component)
        MyBase.EnableDragDrop(False)

        ' Referenz zum bearbeiteten Steuerelement anlegen
        m_AlignContainer = DirectCast(component, AlignContainer)
    End Sub

    ''' <summary>
    ''' Setzt den Wert fr MouseOver und sorgt fr Aktualisierung der Anzeige,
    ''' wenn die Maus in den Bereich des Controls eintritt
    ''' </summary>
    Protected Overrides Sub OnMouseEnter()
        Me.m_mouseover = True
        Me.Control.Refresh()
    End Sub

    ''' <summary>
    ''' Setzt den Wert fr MouseOver und sorgt fr Aktualisierung der Anzeige,
    ''' wenn die Maus den Bereich des Controls verlsst
    ''' </summary>
    Protected Overrides Sub OnMouseLeave()
        Me.m_mouseover = False
        Me.Control.Refresh()
    End Sub

    ''' <summary>
    ''' benutzerdefinierts Zeichnen auf der Control-Flche
    ''' </summary>
    Protected Overrides Sub OnPaintAdornments(ByVal pe As System.Windows.Forms.PaintEventArgs)
        ' Bereich des orangen Rechtecks zur Darstellung von ContentAlignment ermitteln
        Dim AlignRect As New Rectangle(0, 0, 40, 40)
        Select Case m_AlignContainer.ContentAlignment
            Case Drawing.ContentAlignment.TopLeft, Drawing.ContentAlignment.MiddleLeft, Drawing.ContentAlignment.BottomLeft
                AlignRect.X = 0
            Case Drawing.ContentAlignment.TopCenter, Drawing.ContentAlignment.MiddleCenter, Drawing.ContentAlignment.BottomCenter
                AlignRect.X = m_AlignContainer.Width \ 2 - 20
            Case Else
                AlignRect.X = m_AlignContainer.Width - 40
        End Select
        Select Case m_AlignContainer.ContentAlignment
            Case Drawing.ContentAlignment.TopLeft, Drawing.ContentAlignment.TopCenter, Drawing.ContentAlignment.TopRight
                AlignRect.Y = 0
            Case Drawing.ContentAlignment.MiddleLeft, Drawing.ContentAlignment.MiddleCenter, Drawing.ContentAlignment.MiddleRight
                AlignRect.Y = m_AlignContainer.Height \ 2 - 20
            Case Else
                AlignRect.Y = m_AlignContainer.Height - 40
        End Select
        ' oranges Rechteck zeichnen
        pe.Graphics.FillRectangle(Brushes.Orange, AlignRect)

        ' abhngig von CenterSmallVertikal und CenterSmallHorizontal
        ' grn gepunktete zentrierte Rechtecke zeichnen
        If m_AlignContainer.CenterSmallVertikal Then
            pe.Graphics.DrawRectangle(m_DotPen, 0, m_AlignContainer.Height \ 2 - 10, Me.Control.Size.Width, 20)
        End If
        If m_AlignContainer.CenterSmallHorizontal Then
            pe.Graphics.DrawRectangle(m_DotPen, m_AlignContainer.Width \ 2 - 10, 0, 20, Me.Control.Size.Height)
        End If

        ' bei MouseOver weien Rahmen um das Control zeichnen
        If Me.m_mouseover Then
            pe.Graphics.DrawRectangle(New Pen(New SolidBrush(Me.m_OutlineColor), 6), 0, 0, Me.Control.Size.Width, Me.Control.Size.Height)
        End If

    End Sub

    ''' <summary>
    ''' Fgt zur Design-Zeit dem Control eine Property hinzu, die hier die zu verwendende Rahmenfarbe
    ''' enthlt.
    ''' Das DesignOnlyAttribute sorgt dafr, da die OutlineColor-Property nicht durch den 
    ''' Designer serialisiert wird.
    ''' </summary>
    Protected Overrides Sub PreFilterProperties(ByVal properties As System.Collections.IDictionary)
        Dim pd As PropertyDescriptor = TypeDescriptor.CreateProperty(GetType(AlignContainerDesigner), _
        "OutlineColor", GetType(System.Drawing.Color), New Attribute() {New DesignOnlyAttribute(True)})

        properties.Add("OutlineColor", pd)
    End Sub

#End Region



    ''' <summary>
    ''' Aktionsliste fr SmartTag
    ''' </summary>
    Private Class AlignContainerDesignerActionList
        Inherits DesignerActionList

#Region " Private Variablen "
        ' Referenz zum bearbeiteten Basis-Objekt
        Private m_AlignContainer As AlignContainer
#End Region

#Region " Properties "

        Public Property ContentAlignment() As System.Drawing.ContentAlignment
            Get
                Return m_AlignContainer.ContentAlignment
            End Get
            Set(ByVal value As System.Drawing.ContentAlignment)
                EigenschaftSetzen("ContentAlignment", value)
            End Set
        End Property
        Public Property CenterSmallHorizontal() As Boolean
            Get
                Return m_AlignContainer.CenterSmallHorizontal
            End Get
            Set(ByVal value As Boolean)
                EigenschaftSetzen("CenterSmallHorizontal", value)
            End Set
        End Property
        Public Property CenterSmallVertikal() As Boolean
            Get
                Return m_AlignContainer.CenterSmallVertikal
            End Get
            Set(ByVal value As Boolean)
                EigenschaftSetzen("CenterSmallVertikal", value)
            End Set
        End Property

#End Region

#Region " Konstruktor "

        Public Sub New(ByVal Ctl As AlignContainer)
            MyBase.New(Ctl)
            m_AlignContainer = Ctl
        End Sub

#End Region

#Region " Methoden "

        ''' <summary>
        ''' Hilfsmethode zur bernahme von Objekten in Properties des Basis-Objektes
        ''' </summary>
        ''' <param name="Name">Property-Name</param>
        ''' <param name="Wert">zu bernehmender Wert</param>
        Private Sub EigenschaftSetzen(ByVal Name As String, ByVal Wert As Object)
            Dim PD As PropertyDescriptor = TypeDescriptor.GetProperties(m_AlignContainer)(Name)
            If IsNothing(PD) Then
                Throw New ArgumentException("Die Eigenschaft " & Name & " existiert nicht.")
            Else
                ' Wert bergeben
                PD.SetValue(m_AlignContainer, Wert)
                ' Referenziertes Basis-Objekt aktualisieren
                m_AlignContainer.Refresh()
            End If
        End Sub

        Public Overrides Function GetSortedActionItems() As System.ComponentModel.Design.DesignerActionItemCollection
            ' Smart Tag aufbauen
            Dim result As New DesignerActionItemCollection

            ' Sektionsberschrift
            result.Add(New DesignerActionHeaderItem("Darstellung", "Darstellung"))
            ' Eigenschaften
            result.Add(New DesignerActionPropertyItem("ContentAlignment", "ContentAlignment", "Darstellung", "Gibt die Ausrichtung enthaltener Controls zurck oder legt diese fest."))
            ' Eigenschaften
            result.Add(New DesignerActionPropertyItem("CenterSmallHorizontal", "CenterSmallHorizontal", "Darstellung", "Zentriert den Inhalt horizontal, wenn das Control breiter als der Inhalt ist."))
            ' Eigenschaften
            result.Add(New DesignerActionPropertyItem("CenterSmallVertikal", "CenterSmallVertikal", "Darstellung", "Zentriert den Inhalt vertikal, wenn das Control hher als der Inhalt ist."))
            Return result

        End Function
#End Region

    End Class

End Class
