Imports System.Drawing.Drawing2D
Imports system.ComponentModel
''' <summary>
''' ContainerControl mit farbigem Label
''' </summary>
<DesignerAttribute(GetType(ColorLabelContainerDesigner)), DefaultProperty("Text"), ToolboxBitmap(GetType(ColorLabelContainer), "CtlColorLabelContainer.png")> _
Public Class ColorLabelContainer
    Inherits ContainerControl

#Region " Private Variablen "

    ' Farben fr Gradienten und Text
    Private m_GradientColor1 As Color = Color.SteelBlue
    Private m_GradientColor2 As Color = Color.White
    Private m_TextColor As Color = Color.White

#End Region

#Region " Properties "

    <Category("ColorLabelContainer"), Description("Gibt die linke Farbe im Gradienten des Labels zurck oder legt diese fest.")> _
       Property GradientColor1() As Color
        Get
            Return m_GradientColor1
        End Get
        Set(ByVal value As Color)
            m_GradientColor1 = value
            ' nur Label-Rechteck neu zeichnen
            Me.Invalidate(New Rectangle(0, 0, Me.Width, 20))
        End Set
    End Property
    <Category("ColorLabelContainer"), Description("Gibt die rechte Farbe im Gradienten des Labels zurck oder legt diese fest.")> _
    Property GradientColor2() As Color
        Get
            Return m_GradientColor2
        End Get
        Set(ByVal value As Color)
            m_GradientColor2 = value
            Me.Invalidate(New Rectangle(0, 0, Me.Width, 20))
        End Set
    End Property
    <Category("ColorLabelContainer"), Description("Gibt die Textfarbe im Label zurck oder legt diese fest.")> _
    Property TextColor() As Color
        Get
            Return m_TextColor
        End Get
        Set(ByVal value As Color)
            m_TextColor = value
            Me.Invalidate(New Rectangle(0, 0, Me.Width, 20))
        End Set
    End Property
    <Category("ColorLabelContainer"), Description("Der Text, der im Label erscheint.")> _
    Shadows Property Text() As String
        Get
            Return MyBase.Text
        End Get
        Set(ByVal value As String)
            MyBase.Text = value
            Me.Invalidate(New Rectangle(0, 0, Me.Width, 20))
        End Set
    End Property

#End Region

#Region " Konstruktor "

    Public Sub New()
        ' DoubleBuffering zum Vermeiden von Flimmern aktivieren
        Me.SetStyle(ControlStyles.DoubleBuffer Or ControlStyles.UserPaint _
         Or ControlStyles.AllPaintingInWmPaint, True)
        Me.UpdateStyles()
    End Sub

#End Region

#Region " Methoden "
    ' berschreiben der OnControlAdded-Methode der Basisklasse
    Protected Overrides Sub OnControlAdded(ByVal e As System.Windows.Forms.ControlEventArgs)
        MyBase.OnControlAdded(e)

        ' fr die Events LocationChanged und Paint der Child-Controls werden hier
        ' Handler vorgesehen, einmal um den Control-Rahmen bei Notwendigkeit im 
        ' Vordergrund der Children zu zeichnen, zum anderen, um Controls aus dem
        ' Label-Bereich heraus zu halten
        AddHandler e.Control.LocationChanged, AddressOf ChildControl_LocationChanged
        AddHandler e.Control.Paint, AddressOf ChildControl_Paint
    End Sub

    ' berschreiben der OnControlRemoved-Methode der Basisklasse
    Protected Overrides Sub OnControlRemoved(ByVal e As System.Windows.Forms.ControlEventArgs)
        ' Die zuvor hinzu gefgten Eventhandler werden hier wieder vom ehemaligen
        ' Child entfernt
        RemoveHandler e.Control.LocationChanged, AddressOf ChildControl_LocationChanged
        RemoveHandler e.Control.Paint, AddressOf ChildControl_Paint
        MyBase.OnControlRemoved(e)
    End Sub

    ' berschreiben der OnResize-Methode der Basisklasse
    Protected Overrides Sub OnResize(ByVal e As System.EventArgs)
        MyBase.OnResize(e)
        ' nach Resize das ganze Control inclusive Children neu zeichnen
        Invalidate(True)
    End Sub

    ' berschreiben der OnPaint-Methode der Basisklasse
    Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        MyBase.OnPaint(e)
        ' Referenz zum Graphics-Objekt des Controls
        Dim g As Graphics = e.Graphics

        ' GradientBrush erstellen
        Dim GradBrush As New LinearGradientBrush(New Rectangle(0, 0, Me.Width, 20), m_GradientColor1, m_GradientColor2, -35.0F)
        ' 20 Pixel hohes Rechteck mit GradientBrush ausfllen
        g.FillRectangle(GradBrush, New Rectangle(0, 0, Me.Width, 20))
        ' Ressourcen fr GradientBrush frei geben
        GradBrush.Dispose()

        ' Brush fr Text
        Dim TxtBrush As New SolidBrush(m_TextColor)
        ' Text im Label zeichnen
        g.DrawString(Me.Text, Me.Font, TxtBrush, 2, 3)
        ' Ressourcen fr Text-Brush frei geben
        TxtBrush.Dispose()

        ' Rahmen um Control und Label zeichnen
        ControlPaint.DrawVisualStyleBorder(g, New Rectangle(0, 19, Me.Width, Me.Height - 1))
        ControlPaint.DrawVisualStyleBorder(g, New Rectangle(0, 0, Me.Width - 1, Me.Height - 1))
    End Sub

    ' Handler fr das LocationChanged-Event der Children
    Private Sub ChildControl_LocationChanged(ByVal sender As Object, ByVal e As System.EventArgs)
        Dim Ctl As Control = sender
        ' wenn das Child-Control sich im Label-Bereich befindet (20 Pixel hoch),
        ' dann erhlt das Child-Control eine Y-Position von 20
        If Ctl.Location.Y < 20 Then
            Ctl.Location = New Point(Ctl.Location.X, 20)
        End If
    End Sub

    ' Handler fr das Paint-Event der Children
    Private Sub ChildControl_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs)
        ' dies hier ist eigentlich nur notwendig fr Children, die sich im Zeichenbereich des 
        ' Container-Rahmens befinden
        Dim g As Graphics = e.Graphics
        Dim Ctl As Control = sender
        ' Rechteck fr Container-Rahmen aus Child-Sicht
        Dim Rect As New Rectangle(Ctl.Location.X * -1, Ctl.Location.Y * -1, Me.Width - 1, Me.Height - 1)
        ' Container-Rahmen auf Child zeichnen
        ControlPaint.DrawVisualStyleBorder(g, Rect)
    End Sub
#End Region

End Class
