Imports System.ComponentModel
Imports System.ComponentModel.Design
''' <summary>
''' Designer fr einen ColorLabelContainer
''' </summary>
Public Class ColorLabelContainerDesigner
    Inherits System.Windows.Forms.Design.ParentControlDesigner

#Region " Private Variablen "

    ' Referenz zur Aktionsliste fr den SmartTag
    Private m_meineAktionen As DesignerActionListCollection
    ' Referenz zum aktuell bearbeiteten ColorLabelContainer
    Private m_ColorLabelContainer As ColorLabelContainer

#End Region

#Region " Properties "

    ''' <summary>
    ''' Referenz zur Aktionsliste mit Aktionen, die der SmartTag beinhalten soll
    ''' </summary>
    Public Overrides ReadOnly Property ActionLists() As System.ComponentModel.Design.DesignerActionListCollection
        Get
            If m_meineAktionen Is Nothing Then
                ' Aktionsliste wird bei Bedarf einmalig als Objekt instanziiert
                m_meineAktionen = New DesignerActionListCollection
                Dim Liste As New ColorLabelContainerDesignerActionList(CType(m_ColorLabelContainer, Control))
                m_meineAktionen.Add(Liste)
            End If
            Return m_meineAktionen
        End Get
    End Property

#End Region

#Region " Methoden "
    ''' <summary>
    ''' Initialisiert den Designer.
    ''' </summary>
    ''' <param name="component">Zugehrige Komponente.</param>
    Public Overloads Overrides Sub Initialize(ByVal component As IComponent)
        ' Basisklasse initialisieren
        MyBase.Initialize(component)
        MyBase.EnableDragDrop(False)

        ' Referenz zum bearbeiteten Steuerelement anlegen
        m_ColorLabelContainer = DirectCast(component, ColorLabelContainer)
    End Sub

#End Region


    ''' <summary>
    ''' Aktionsliste fr SmartTag
    ''' </summary>
    Private Class ColorLabelContainerDesignerActionList
        Inherits DesignerActionList

#Region " Private Variablen "
        ' Referenz zum bearbeiteten Basis-Objekt
        Private m_ColorLabelContainer As ColorLabelContainer
#End Region

#Region " Properties "

        Property GradientColor1() As Color
            Get
                Return m_ColorLabelContainer.GradientColor1
            End Get
            Set(ByVal value As Color)
                EigenschaftSetzen("GradientColor1", value)
            End Set
        End Property
        Property GradientColor2() As Color
            Get
                Return m_ColorLabelContainer.GradientColor2
            End Get
            Set(ByVal value As Color)
                EigenschaftSetzen("GradientColor2", value)
            End Set
        End Property
        Property TextColor() As Color
            Get
                Return m_ColorLabelContainer.TextColor
            End Get
            Set(ByVal value As Color)
                EigenschaftSetzen("TextColor", value)
            End Set
        End Property
        Property Text() As String
            Get
                Return m_ColorLabelContainer.Text
            End Get
            Set(ByVal value As String)
                EigenschaftSetzen("Text", value)
            End Set
        End Property

#End Region

#Region " Konstruktor "

        Public Sub New(ByVal Ctl As ColorLabelContainer)
            MyBase.New(Ctl)
            m_ColorLabelContainer = Ctl
        End Sub

#End Region

#Region " Methoden "

        ''' <summary>
        ''' Hilfsmethode zur bernahme von Objekten in Properties des Basis-Objektes
        ''' </summary>
        ''' <param name="Name">Property-Name</param>
        ''' <param name="Wert">zu bernehmender Wert</param>
        Private Sub EigenschaftSetzen(ByVal Name As String, ByVal Wert As Object)
            Dim PD As PropertyDescriptor = TypeDescriptor.GetProperties(m_ColorLabelContainer)(Name)
            If IsNothing(PD) Then
                Throw New ArgumentException("Die Eigenschaft " & Name & " existiert nicht")
            Else
                ' Wert bergeben
                PD.SetValue(m_ColorLabelContainer, Wert)
                ' Referenziertes Basis-Objekt aktualisieren
                m_ColorLabelContainer.Refresh()
            End If
        End Sub

        Public Overrides Function GetSortedActionItems() As System.ComponentModel.Design.DesignerActionItemCollection
            ' Smart Tag aufbauen
            Dim result As New DesignerActionItemCollection

            ' Sektionsberschrift
            result.Add(New DesignerActionHeaderItem("Darstellung", "Darstellung"))
            ' Eigenschaften
            result.Add(New DesignerActionPropertyItem("GradientColor1", "GradientColor1", "Darstellung", "Linke Farbe im Gradienten des Labels."))
            result.Add(New DesignerActionPropertyItem("GradientColor2", "GradientColor2", "Darstellung", "Rechte Farbe im Gradienten des Labels."))
            result.Add(New DesignerActionPropertyItem("TextColor", "TextColor", "Darstellung", "Textfarbe im Label."))
            result.Add(New DesignerActionPropertyItem("Text", "Text", "Darstellung", "Der Text, der im Label erscheint."))

            Return result

        End Function

#End Region

    End Class

End Class
